# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: release-1.29
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kubernetes.client.configuration import Configuration


class V1alpha2ResourceClaimConsumerReference(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'api_group': 'str',
        'name': 'str',
        'resource': 'str',
        'uid': 'str'
    }

    attribute_map = {
        'api_group': 'apiGroup',
        'name': 'name',
        'resource': 'resource',
        'uid': 'uid'
    }

    def __init__(self, api_group=None, name=None, resource=None, uid=None, local_vars_configuration=None):  # noqa: E501
        """V1alpha2ResourceClaimConsumerReference - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._api_group = None
        self._name = None
        self._resource = None
        self._uid = None
        self.discriminator = None

        if api_group is not None:
            self.api_group = api_group
        self.name = name
        self.resource = resource
        self.uid = uid

    @property
    def api_group(self):
        """Gets the api_group of this V1alpha2ResourceClaimConsumerReference.  # noqa: E501

        APIGroup is the group for the resource being referenced. It is empty for the core API. This matches the group in the APIVersion that is used when creating the resources.  # noqa: E501

        :return: The api_group of this V1alpha2ResourceClaimConsumerReference.  # noqa: E501
        :rtype: str
        """
        return self._api_group

    @api_group.setter
    def api_group(self, api_group):
        """Sets the api_group of this V1alpha2ResourceClaimConsumerReference.

        APIGroup is the group for the resource being referenced. It is empty for the core API. This matches the group in the APIVersion that is used when creating the resources.  # noqa: E501

        :param api_group: The api_group of this V1alpha2ResourceClaimConsumerReference.  # noqa: E501
        :type: str
        """

        self._api_group = api_group

    @property
    def name(self):
        """Gets the name of this V1alpha2ResourceClaimConsumerReference.  # noqa: E501

        Name is the name of resource being referenced.  # noqa: E501

        :return: The name of this V1alpha2ResourceClaimConsumerReference.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this V1alpha2ResourceClaimConsumerReference.

        Name is the name of resource being referenced.  # noqa: E501

        :param name: The name of this V1alpha2ResourceClaimConsumerReference.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def resource(self):
        """Gets the resource of this V1alpha2ResourceClaimConsumerReference.  # noqa: E501

        Resource is the type of resource being referenced, for example \"pods\".  # noqa: E501

        :return: The resource of this V1alpha2ResourceClaimConsumerReference.  # noqa: E501
        :rtype: str
        """
        return self._resource

    @resource.setter
    def resource(self, resource):
        """Sets the resource of this V1alpha2ResourceClaimConsumerReference.

        Resource is the type of resource being referenced, for example \"pods\".  # noqa: E501

        :param resource: The resource of this V1alpha2ResourceClaimConsumerReference.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and resource is None:  # noqa: E501
            raise ValueError("Invalid value for `resource`, must not be `None`")  # noqa: E501

        self._resource = resource

    @property
    def uid(self):
        """Gets the uid of this V1alpha2ResourceClaimConsumerReference.  # noqa: E501

        UID identifies exactly one incarnation of the resource.  # noqa: E501

        :return: The uid of this V1alpha2ResourceClaimConsumerReference.  # noqa: E501
        :rtype: str
        """
        return self._uid

    @uid.setter
    def uid(self, uid):
        """Sets the uid of this V1alpha2ResourceClaimConsumerReference.

        UID identifies exactly one incarnation of the resource.  # noqa: E501

        :param uid: The uid of this V1alpha2ResourceClaimConsumerReference.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and uid is None:  # noqa: E501
            raise ValueError("Invalid value for `uid`, must not be `None`")  # noqa: E501

        self._uid = uid

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1alpha2ResourceClaimConsumerReference):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1alpha2ResourceClaimConsumerReference):
            return True

        return self.to_dict() != other.to_dict()
